#include "grbc/ext_pkg_config.h"
#include "grbc/ext.h"
#include "grbc/helpers.h"
#include "grbc/spec.h"
#include "sol/state.hpp"
#include <cstdlib>

std::string pkg_config_util_read_file(FILE *file) {
  // http://www.fundza.com/c4serious/fileIO_reading_all/index.html

  char line[190];
  std::string result;

  while (fgets(line, 190, file))
    result += line;

  if (result.back() == '\n')
    result.pop_back();

  return result;
}

Package EXT_pkg_config_get_pkg(const std::string &name) {
  int found = std::system(("pkg-config " + name).c_str());

  if (found != EXIT_SUCCESS) {
    grbc_exception(
        ("Failed to find package using pkg-config: " + name).c_str());
  }

  Package pkg{};

  FILE *f_cflags = popen(("pkg-config --cflags " + name).c_str(), "r");
  FILE *f_libs = popen(("pkg-config --libs " + name).c_str(), "r");

  std::string compiler_flags = pkg_config_util_read_file(f_cflags);
  std::string linker_flags = pkg_config_util_read_file(f_libs);

  fclose(f_cflags);
  fclose(f_libs);

  pkg.name = name;
  pkg.compiler_flags = compiler_flags;
  pkg.linker_flags = linker_flags;

  return pkg;
}

void grbc_pkg_config_init(sol::state &state) {
  state.set("grbc_pkg_config", EXT_pkg_config_get_pkg);

  log_msg("pkg-config supported loaded!");
}

Extension grbc_pkg_config() {
  Extension ext{};
  ext.name = GRBC_EXT_pkg_config_NAME;
  ext.hook_init = grbc_pkg_config_init;

  return ext;
}