#include "grbc/generator.h"
#include "grbc/spec.h"
#include "grbc/state.h"

TargetInfo grbc_executable(const ExecutableConfig &executable_config) {
  TargetInfo target_config{};

  std::string exe_name = executable_config.name;

  if (grbc_is_win32())
    exe_name += ".exe";

  std::string compiler_args;
  std::string linker_args;

  for (auto &package : executable_config.requirements) {
    compiler_args += package.compiler_flags + " ";
    linker_args += package.linker_flags + " ";
  }

  for (auto &linker_arg : executable_config.linker_flags) {
    linker_args += linker_arg + " ";
  }

  for (auto &compiler_arg : executable_config.compile_flags) {
    compiler_args += compiler_arg + " ";
  }

  // Include directories

  compiler_args += grbc_include_dirs_to_cflags(executable_config.include_dirs);

  std::vector<std::string> libraries;

  for (auto &lib : executable_config.requirements) {
    libraries.push_back(lib.file_name);
  }

  GeneratorTarget target{};

  std::vector<std::string> object_files;

  for (auto &src_file : executable_config.files) {
    GeneratorCompileCommand compile_cmd{};
    compile_cmd.source_file = src_file;
    compile_cmd.object_file = grbc_object_file(src_file);
    compile_cmd.compiler_flags = compiler_args;
    compile_cmd.language_type = executable_config.language_type;

    object_files.push_back(compile_cmd.object_file);

    target.compile_commands.push_back(compile_cmd);
  }

  // Final executable link

  GeneratorLinkTargetCommand executable_link_cmd{};
  executable_link_cmd.linker_flags = linker_args;
  executable_link_cmd.object_files = object_files;
  executable_link_cmd.output_name = exe_name;
  executable_link_cmd.target_type = GeneratorTargetType_Executable;
  executable_link_cmd.libraries = libraries;

  target.link_target_commands.push_back(executable_link_cmd);

  GState::get().targets.push_back(target);

  return target_config;
}